@@ -3,6 +3,7 @@
 import sys, os, os.path as osp, shutil, tempfile, time, re
 from contextlib import contextmanager
 from urllib import quote_plus
+from urlparse import urlparse
 
 from mercurial import commands, hg
 
@@ -80,11 +81,20 @@ class HGSRepository(Repository):
             commands.pull(repo.ui, repo,
                           source=self.source_url.encode('ascii'))
             commands.update(repo.ui, repo)
-            yield repo.ui, repo
+            yield repo
             commands.commit(repo.ui, repo, **commitopts)
             commands.push(repo.ui, repo,
                           dest=self.source_url.encode('ascii'))
 
+    @property
+    def push_url(self):
+        if self.source_url.startswith(self.hosted_by[0].base_url + '/'):
+            return self.source_url
+        # XXX this assumes that the root of the (probably) http source_url is
+        # the same as the mercurial-server base_url
+        path = urlparse(self.source_url).path
+        return self.hosted_by[0].base_url + path
+
 
 class MercurialServerConfig(AnyEntity):
     __regid__ = 'MercurialServerConfig'
@@ -109,8 +119,8 @@ class MercurialServerConfig(AnyEntity):
 
     @contextmanager
     def exec_in_hgadmin_clone(self, message, **commitopts):
-        with self.hgadmin_repo.pull_push(message, **commitopts) as (ui, repo):
-            yield ui, repo
+        with self.hgadmin_repo.pull_push(message, **commitopts) as repo:
+            yield repo
 
     def write_access_conf(self, ui, repo, access_line):
         access_conf_path = self.access_conf_path(repo)
@@ -122,8 +132,8 @@ class MercurialServerConfig(AnyEntity):
         """ Generate the contents of the access.conf file """
         access_conf = []
         for repo in self.reverse_hosted_by:
-            assert repo.source_url.startswith(self.base_url + '/')
-            repo_path = repo.source_url[len(self.base_url) + 1:]
+            assert repo.push_url.startswith(self.base_url + '/')
+            repo_path = repo.push_url[len(self.base_url) + 1:]
             for perm in repo.reverse_granted_on:
                 key_path = perm.access_key[0].pathtuple
                 conf_line = '{level} user={key} repo={repo}'.format(
@@ -137,6 +147,10 @@ class MercurialServerConfig(AnyEntity):
 class MercurialServerPermission(AnyEntity):
     __regid__ = 'MercurialServerPermission'
 
+    def dc_title(self):
+        return '%s for %s' % (self.permission_level,
+                              self.access_key[0].reverse_public_key[0].dc_title())
+
     @property
     def pubkey(self):
         return self.access_key[0].data
