@@ -22,7 +22,7 @@ import os.path as osp
 import shutil
 from collections import defaultdict
 
-from mercurial import commands, ui, hg, error
+from mercurial import commands, hg, error
 
 from cubicweb import ValidationError
 from cubicweb.predicates import is_instance, match_user_groups
@@ -33,7 +33,7 @@ from cubes.mercurial_server.utils import mscui
 
 # MercurialServerConfig
 
-class MercurialServerConfig(Hook):
+class MSCCreateHook(Hook):
     """ MSC creation -> automatically create its hgadmin Repository
     Deletion: the mercurial-server data will be out-of sync until
     a new MSC is created again, which will reset its keys & configs.
@@ -48,6 +48,7 @@ class MercurialServerConfig(Hook):
         hgadmin_path = edit.get('hgadmin_path')
         if not base_url or not hgadmin_path:
             return # standard validation will catch this, let's bail out
+        base_url = edit['base_url'] = base_url.rstrip('/')
         url = '/'.join((base_url, hgadmin_path))
         repo = self._cw.create_entity('Repository',
                                       title=u'%s_repository_for_%s' % (hgadmin_path,
@@ -58,6 +59,19 @@ class MercurialServerConfig(Hook):
         edit['hgadmin_repository'] = repo.eid
 
 
+class MSCUpdateHook(Hook):
+    """strip trailing /s from base_url, if modified"""
+    __regid__ = 'mercurialserver.config.update'
+    __select__ = Hook.__select__ & is_instance('MercurialServerConfig')
+    events = ('before_update_entity',)
+
+    def __call__(self):
+        edit = self.entity.cw_edited
+        if 'base_url' not in edit:
+            return
+        edit['base_url'] = edit['base_url'].rstrip('/')
+
+
 class MercurialServerRepositoryCreation(Hook):
     """ When a Repository is linked to a MercurialServerConfig
     through hosted_by, we try to initialize its remote location
@@ -70,28 +84,29 @@ class MercurialServerRepositoryCreation(Hook):
     category = 'mercurialserver.fs'
 
     def __call__(self):
-        user = self._cw.user
-        repoeid = self.eidfrom
-        repo = self._cw.entity_from_eid(repoeid)
-        if (user.eid not in set(owner.eid for owner in repo.owned_by) and
-            'managers' not in user.groups):
-            msg = self._cw._('Only managers or owners can set a repository to a '
-                             'mercurial-server host.')
-            raise ValidationError(self.eidfrom, {'hosted_by': msg})
+        # we need to refresh user's relations cache and cw_clear_cache is not
+        # thread safe while _cw.user is a shared object. Prefer this method.
+        user = self._cw.entity_from_eid(self._cw.user.eid)
+        repo = self._cw.entity_from_eid(self.eidfrom)
         if not user.public_key:
             msg = (self._cw._('To create a hosted repository, %s must have a public key')
                    % user.login)
             raise ValidationError(self.eidfrom, {'hosted_by': msg})
         try:
-            uri = repo.source_url.encode('ascii')
+            uri = repo.push_url.encode('ascii')
         except UnicodeEncodeError:
             msg = self._cw._('source_url must be ASCII encodable')
             raise ValidationError(self.eidfrom, {'source_url': msg})
+        hgui = mscui()
         try:
-            commands.init(ui.ui(), uri)
+            commands.init(hgui, uri)
             self.warning('init-ed repository %s', uri)
         except error.RepoError:
-            self.warning('repo %s seems to be already there: cannot init', uri)
+            try:
+                commands.identify(hgui, None, source=uri)
+                self.warning('repo %s seems to be already there: cannot init', uri)
+            except error.RepoError:
+                raise ValidationError(self.eidfrom, {'source_url': hgui.popbuffer()})
         else:
             # prepare possible rollback (destroy remote repository)
             MSRepoRollbackOnInit.get_instance(self._cw).add_data(uri)
@@ -272,12 +287,12 @@ def sync_access_keys(session, server_config, op_path_pubkey):
     msg = ('Access keys sync from cube mercurial-server for %s' %
            ', '.join([x[1] for x in op_path_pubkey]))
     userdesc = session.user.format_name_for_mercurial()
-    with server_config.exec_in_hgadmin_clone(msg, user=userdesc) as (ui, repo):
+    with server_config.exec_in_hgadmin_clone(msg, user=userdesc) as repo:
         for op, path, pubkey in op_path_pubkey:
             if op == 'add':
-                add_key(session, ui, repo, path, pubkey)
+                add_key(session, repo.ui, repo, path, pubkey)
             else:
-                remove_key(session, ui, repo, path)
+                remove_key(session, repo.ui, repo, path)
 
 
 class SynchronizeAccesKeysOperation(DataOperationMixIn, Operation):
@@ -303,7 +318,7 @@ class SynchronizeAccesKeysOperation(DataOperationMixIn, Operation):
 def write_access_conf(session, msc):
     msg = ('Regenerate access configuration %s from cube mercurial-server' %
            msc.dc_title())
-    with msc.exec_in_hgadmin_clone(msg) as (ui_, repo):
+    with msc.exec_in_hgadmin_clone(msg) as repo:
         access_conf_contents = msc.generate_access_conf()
         filename = osp.join(repo.root, msc.conf_filename_str)
         already_exists = osp.exists(filename)
@@ -311,7 +326,7 @@ def write_access_conf(session, msc):
             fobj.write(access_conf_contents)
         if not already_exists:
             session.warning('write %s', filename)
-            commands.add(ui_, repo, filename)
+            commands.add(repo.ui, repo, filename)
 
 class RegenerateAccesConfOperation(DataOperationMixIn, Operation):
     """ Regenerate the mercurial-server <access.conf> file"""
@@ -351,9 +366,9 @@ class RegenerateMSCRepoService(Service):
     def call(self, eid):
         msc = self._cw.entity_from_eid(eid)
         # first, clean
-        msg = "Regenrate cfg and keys"
+        msg = "Regenerate cfg and keys"
         userdesc = self._cw.user.format_name_for_mercurial()
-        with msc.exec_in_hgadmin_clone(msg, user=userdesc) as (ui, hgrepo):
+        with msc.exec_in_hgadmin_clone(msg, user=userdesc) as hgrepo:
             commands.remove(hgrepo.ui, hgrepo,
                             osp.join(hgrepo.root, 'keys', msc.keys_subdir))
             RegenerateAccesConfOperation.get_instance(self._cw).add_data(eid)
@@ -367,6 +382,6 @@ class RegenerateMSCRepoService(Service):
                     os.makedirs(dirpath)
                 with open(filename, 'wb') as keyfile:
                     keyfile.write(key.data.getvalue())
-                commands.add(ui, hgrepo, str(filename))
+                commands.add(hgrepo.ui, hgrepo, str(filename))
         # ... just in case, we don't really know who is responsible for this
         self._cw.commit()
